function dx = vmodel_bicycle_lincontrol_ff(t,x,closedloop)
% vmodel_bicycle_lincontrol_ff - linear feed-back, nonlinear (I/Olin) feed-forward 
% author: Daniel.Hess@DLR.de
% input:
%         x = [ state
%              1:6   [X,Y,PSI,vx,vy,dPSI], contains Positions/Velocities of COR of *real vehicle* 
%              7:8   [e_psi,e_omega], contains internal dynamics states of *reference vehicle* 
%              9:15  [Xs,Ys,Thetas,vs,kappas,dvs,dkappasds], contains state of set trajectory 
%         err = [e_fx,e_fyf,e_fyr]
% output:
%         [ax,delta] control input generated by closed loop controller and fed into the system 
% - hess_da, 05.04.2016 - the integral part 7:8 is deactivated


    %% vehicle parameters: p.47, tab. 5
    Iz_m =    1.1700; %Iz/m
    b_L  =    0.4231; %b/L
    cf   = -  5.7840; %-10.8; 
    cr   = - 17.1630;%-17.8;
    h    =    0.4000;
    L    =    1.6860;
    g    =    9.8100;
    mu   =    0.8000;
    b    = b_L * L;
    a    = (1 - b_L) * L;
    m_J  = 1 / Iz_m;
    
    %% controller parameters:
    lff = 0;%0;%Iz_m/b;%a % the position, which should follow the set trajectory (also defines internal dynamics damping of *reference vehicle*) 
%     lx = 2.1; % look ahead of linear controller --> defines internal dynamics of *real vehicle* 
    lx = 7; % look ahead of linear controller --> defines internal dynamics of *real vehicle* 
%     kx = [2,4,3]; %linear controller feed-back gain, I-P-D; place([0,1,0;0,0,1;0,0,0],[0;0;1],[-1+1i,-1-1i,-1])
%     kx = [0,2,2];%place([0,1,0;0,0,1;0,0,0],[0;0;1],[-1+1i,-1-1i,0])
    kx = [0,4.5,3];
%     kx = [0,6,5];%place([0,1,0;0,0,1;0,0,0],[0;0;1],[-2,-3,0])
     kx = [0,0,0];
    if x(4)<1
        v0 = 1;
        steering_gain = mu*b/L*g*(-cf)*(v0+a*lx);% ay_lx = steering_gain * delta; dvy+lx*domega -->(1+a*lx)
    else
        steering_gain = mu*b/L*g*(-cf)/x(4)*(1+a*lx);% ay_lx = steering_gain * delta; dvy+lx*domega -->(1+a*lx)
    end
%     ky = [0.1500    1.7500    2.6000]/steering_gain;%factor for converting to lat acc
%     ky = [0,2,2]/steering_gain;%place([0,1,0;0,0,1;0,0,0],[0;0;1],[-1+1i,-1-1i,0])
%     ky = [0,4.5,3]/steering_gain;
    ky = [0,20,2];%/steering_gain;%hess_da, 11.04.2016 - trying to remove influence of vy     
    ky = [0,+ 80.00,-0.000]; % Gains for the lateral action
    kx = [0,+ 0.130,+0.500];
    if closedloop == 0
    	kx = [0,0,0];
    	ky = [0,0,0];
    end
    %% variables 
    X = x(1);Y = x(2);PSI = x(3);vx = x(4);vy = x(5);omega = x(6);
    e_psi = x(7);e_omega = x(8);
    Xs = x(9); Ys = x(10); thetas = x(11);vs = x(12);
    kappas = x(13); dvs = x(14); dkappasds = x(15);
    ie = [0;0];

    %% feed-forward: compute deltaff, axff, de_omega, Xff,Yff,dXff,dYff,thetaff 
    w = R(-e_psi)*[dvs;vs^2*kappas];                                        %set acc. from set trajectory
    psii = e_psi + thetas;                                                  %ref veh yaw
    omegai = e_omega + vs*kappas;                                           %ref veh yaw rate
    axff = w(1) + lff*omegai^2;                                             %feed-forward/ref veh x acc
    vxi = vs*cos(-e_psi);                                                   %ref veh x vel 
    vyi = vs*sin(-e_psi) - lff*omegai;                                      %ref veh y vel @COG
    if vxi<1 % no feed-forward for very low velocities
        deltaff = 0;
        fyri = mu * (a/L*g+h/L*0)*cr*(vyi-b*omegai);                     %ref veh rear lat force/m
        fyfi = ( w(2) - ( 1 - m_J*lff*b ) * fyri ) / ( 1 + m_J*lff*a );     %necessary ref veh front lat force/m to keep to set trajectory 
%         fyri = mu * (a/L*g+h/L*axff)*cr*(vyi-b*omegai);                     %ref veh rear lat force/m
%         fyfi = ( w(2) - ( 1 - m_J*lff*b ) * fyri ) / ( 1 + m_J*lff*a );     %necessary ref veh front lat force/m to keep to set trajectory 
    else
        fyri = mu * (a/L*g+h/L*0)*cr*(vyi-b*omegai)/vxi;                 %ref veh rear lat force/m
        fyfi = ( w(2) - ( 1 - m_J*lff*b ) * fyri ) / ( 1 + m_J*lff*a );     %necessary ref veh front lat force/m to keep to set trajectory 
        alphafi = fyfi / ( mu * (b/L*g-h/L*0)*cf );                      %fyfi = mu * (b/L*g-h/L*u(1))*cf*alpha_f
%         fyri = mu * (a/L*g+h/L*axff)*cr*(vyi-b*omegai)/vxi;                 %ref veh rear lat force/m
%         fyfi = ( w(2) - ( 1 - m_J*lff*b ) * fyri ) / ( 1 + m_J*lff*a );     %necessary ref veh front lat force/m to keep to set trajectory 
%         alphafi = fyfi / ( mu * (b/L*g-h/L*axff)*cf );                      %fyfi = mu * (b/L*g-h/L*u(1))*cf*alpha_f
        deltaff = (vyi+a*omegai)/vxi-alphafi;                               %feed-forward steering angle; alpha = atan2(vy+a*omega,vx)-delta
    end
    domegai = m_J * ( a * fyfi - b * fyri );                                %ref veh internal dynamics input
    
    
    
    %% feed-back
     e = R(-psii)*( [X;Y]   + [cos(PSI);sin(PSI)]   * (lx+b)...               %error of real veh tracking the ref veh
                 -( [Xs;Ys] + [cos(psii);sin(psii)] * (lx-lff) ));                                              
    de = R(PSI-psii)*[vx;vy+(b+lx)*omega] ...                               %derivative of tracking error;
                   - [vxi;vyi+lx*omegai] ...
                   - S(omegai)*e;
    ax    = - kx(1)*ie(1) - kx(2)*e(1) - kx(3)*de(1);                       %linear feedback long-direction
    ayfb = - ky(1)*ie(2) - ky(2)*e(2) - ky(3)*de(2);                       %linear feedback lat-direction
    aymax = 20;
    ayfb = max(-aymax,min(ayfb,aymax));
    delta = ayfb/steering_gain;
    ax = ax + axff;
    delta = delta + deltaff;
    
    %% vehicle model (for simulation)
    ax = ax;
    alpha_f = (vy+(a+b)*omega)/vx - delta;                                  %real veh slip angle front
    alpha_r = vy/vx;                                                        %real veh slip angle rear
    Fyf_Fzf = mu * (b/L*g-h/L*0)*cf*alpha_f;           %real veh normalized front force
    Fyr_Fzr = mu * (a/L*g+h/L*0)*cr*alpha_r;           %real veh normalized rear force
%     Fyf_Fzf = mu * (b/L*g-h/L*ax)*cf*alpha_f            + err(2);           %real veh normalized front force
%     Fyr_Fzr = mu * (a/L*g+h/L*ax)*cr*alpha_r;           + err(3);           %real veh normalized rear force
    ay = Fyf_Fzf + Fyr_Fzr;                                                 %real veh lat acc
    domega = m_J * ( a*Fyf_Fzf - b*Fyr_Fzr );                               %real veh yaw rate der

    %% state derivative
    dx=x;%same size and type
    % real veh state der
    dx(1) = cos(PSI)*vx - sin(PSI)*vy;
    dx(2) = sin(PSI)*vx + cos(PSI)*vy;
    dx(3) = omega;
    dx(4) = ax + (vy+b*omega)*omega;
    dx(5) = ay - vx*omega -b*domega;              
    dx(6) = domega;
    % internal dynamics of ref veh
    dx(7) = e_omega;
    dx(8) = domegai-(dvs*kappas+vs^2*dkappasds);
    % set trajectory
    dx(9) = cos(thetas)*vs;
    dx(10) = sin(thetas)*vs;
    dx(11) = vs*kappas;
    dx(12) = dvs;
    dx(13) = vs*dkappasds;
    dx(14) = 0;
    dx(15) = 0;
    
                   
end


function M = R(alpha)
    M = [cos(alpha),-sin(alpha);sin(alpha),cos(alpha)];
end

function M = S(alpha)
    M = [0,-alpha;alpha,0];
end